/*  Copyright (C) MOXA Inc. All rights reserved.

    This software is distributed under the terms of the
    MOXA License.  See the file COPYING-MOXA for details.
*/

#include "stdafx.h"
#include <winsock2.h>

#define SERVER_SOCKET_ERROR 1
#define SOCKET_OK 0

#define BUFF_SIZE 4096

void socketError(char*);

int _tmain(int argc, _TCHAR* argv[])
{
	WORD sockVersion;
	WSADATA wsaData;
	int rVal;

	sockVersion = MAKEWORD(1,1);
	//start dll
	WSAStartup(sockVersion, &wsaData);

	//create socket
	SOCKET s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);

	if(s == INVALID_SOCKET)
	{
		socketError("Failed socket()");
		WSACleanup();
		return SERVER_SOCKET_ERROR;
	}

	//fill in sockaddr_in struct 

	SOCKADDR_IN sin;
	sin.sin_family = PF_INET;
	sin.sin_port = htons(8888);
	sin.sin_addr.s_addr = INADDR_ANY;

	//bind the socket
	rVal = bind(s, (LPSOCKADDR)&sin, sizeof(sin));
	if(rVal == SOCKET_ERROR)
	{
		socketError("Failed bind()");
		WSACleanup();
		return SERVER_SOCKET_ERROR;
	}

	//get socket to listen 
	rVal = listen(s, 2);
	if(rVal == SOCKET_ERROR)
	{
		socketError("Failed listen()");
		WSACleanup();
		return SERVER_SOCKET_ERROR;
	}

	//wait for a client
	SOCKET client;
	//cout << "waiting for newclient" << endl;

	client = accept(s, NULL, NULL);

	//cout << "newclient found" << endl;

	if(client == INVALID_SOCKET)
	{
		socketError("Failed accept()");
		WSACleanup();
		return SERVER_SOCKET_ERROR;
	}
	
	char bufcmp[BUFF_SIZE];
	for (int m=0; m<BUFF_SIZE; m++)
	{
		bufcmp[m] = m %256;
	}
	char buffer[BUFF_SIZE];
	fd_set				readfds;
	struct timeval		authtime;
	unsigned int nTotal = 0, nIndex=0;
	bool bError=FALSE;
	while (1)
	{
		//char *data = readline(&client);
		memset(buffer,0,sizeof(buffer));
		authtime.tv_usec = 1000L;
		authtime.tv_sec = 0;
		FD_ZERO(&readfds);
		FD_SET(client, &readfds);
		int s = (int)client+1;
		if (select (s, &readfds, NULL, NULL, &authtime) >= 0) 
		{
			if (FD_ISSET (client, &readfds)) 
			{
				rVal = recv(client, buffer, BUFF_SIZE, 0);
				if(rVal == SOCKET_ERROR)
				{
					int errorVal = WSAGetLastError();

					if(errorVal == WSAENOTCONN)
					{
						socketError("Socket not connected!");
					}
					socketError("Failed recv()");
					WSACleanup();
					break;
				}
				bError = FALSE;
				int n=0;
				for (n=0; n<rVal; n++)
				{
					if ( buffer[n] != bufcmp[nIndex] )
					{
						bError = TRUE;
						break;
					}
					nIndex++;
					if (nIndex==BUFF_SIZE) {
						nIndex=0;
					}
				}
				if (!bError) {
					nTotal += rVal;
					printf("\r%d  OK", nTotal);
				} else {
					nTotal += n;
					printf("\r%d  Failed", nTotal);
					break;
				}
				
				/*
				if(buffer[0] != 'q')
				{
					printf("[%s]\n", buffer);
				} else {
					break;
				}*/
			}
		}
	}

	//close process
	closesocket(client);
	closesocket(s);

	WSACleanup();
	//cout << "closing down"<< endl;
	return SOCKET_OK;
}

void socketError(char* str)
{
	printf("%s:%s\n", "SERVER SOCKET ERROR", str);
}
