// gprs_diagnose.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "moxa\devices.h"

void PrintUsage( void )
{
	wprintf( TEXT("Usage: gprs_diagnose      ; trouble shooting for the GPRS and SIM card\r\n") );
	wprintf( TEXT("\r\n"));
}


BOOL getopt(int argc, _TCHAR* argv[], _TCHAR option, _TCHAR** value)
{
	int		i;
	BOOL	bolFound = FALSE;

	for (i = 1; (i < argc); i++)
	{
		if ( bolFound )
		{
			if ( value ) {
				*value = argv[i];
			}
			break;
		}
		if ((argv[i][0] == '-') || (argv[i][0] == '/'))
		{
			if ( argv[i][1] == option )
			{
				bolFound = TRUE;
			}
		}
	}

	return bolFound;
}

BOOL ShowBand(int nBand)
{
	switch(nBand)
	{
		case 1:
			printf( "GSM 900 MHz\n");
			return TRUE;
		case 2:
			printf( "GSM 1800 MHz\n");
			return TRUE;
		case 3:
			printf( "GSM 900 MHz + GSM 1800 MHz\n");
			return TRUE;
		case 4:
			printf( "GSM 1900 MHz\n");
			return TRUE;
		case 5:
			printf( "GSM 900 MHz + GSM 1900 MHz\n");
			return TRUE;
		case 8:
			printf( "GSM 850 MHz\n");
			return TRUE;
		case 10:
			printf( "GSM 850 MHz + GSM 1800 MHz\n");
			return TRUE;
		case 12:
			printf( "GSM 850 MHz + GSM 1900 MHz\n");
			return TRUE;
		case 15:
			printf( "GSM 900 MHz + GSM 1800 MHz + GSM 850 MHz + GSM 1900 MHz\n");
			return TRUE;

		default:
			printf( "Unknown Band=%d\n");
			return FALSE;
	}
}

extern int Get_Selected_Radio_Band(HANDLE h);

int _tmain(int argc, _TCHAR* argv[])
{
	int nRet = -1;
	int nSignal=0;

	if ( getopt(argc, argv, 'h', NULL) ) {
		PrintUsage();
		return 0;
	}

	int h = cellular_modem_open();
	if ( h != 0 && h != -1 ) 
	{
		nRet = cellular_modem_gprs_diagnose_status(h);

		if ( nRet & GPRS_ERROR_BAUDRATE_COM3 ) {
			printf("Can't communicate with COM3:\n");
		}
		if ( nRet & GPRS_ERROR_BAUDRATE_COM4 ) {
			printf("Can't communicate with COM4:\n");
		}
		if ( nRet & GPRS_ERROR_SIGNAL_STRENGTH ) {
			printf("Get signal strength error.\n");
		} else {
			nSignal = cellular_modem_gprs_get_signal_strength(h);
			printf("GPRS Signal Strength=%d\n", nSignal);
		}
		if ( nRet & GPRS_ERROR_FLOWCONTROL ) {
			printf("Set GPRS Flow Control error.\n");
		}
		if ( nRet & GPRS_ERROR_PINCODE ) {
			//printf("Need PIN code\n");
		}
		if ( nRet & GPRS_ERROR_TEMPERATURE ) {
			printf("Over Operating temperature.\n");
		} else {
			printf("GPRS module temperature is %d.\n", cellular_modem_gprs_get_module_temperature(h) );
		}
		if ( nRet & GPRS_ERROR_RADIOBAND ) {
			printf("Radio Band Error.\n");
		} else {
			int nBand = Get_Selected_Radio_Band((HANDLE)h);
			printf("Preferred Band=");
			ShowBand(nBand);
		}
		if ( nRet & GPRS_ERROR_MODULE ) {
			printf("GPRS module is not started.\n");
		}
		cellular_modem_close(h);
	}

	return nRet;
}

