// sim_change_pin.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "moxa\devices.h"

void PrintUsage( void )
{
	wprintf( TEXT("Usage: sim_change_pin [-h] | [-o old_pin] [-n new_pin]   ; Change the PIN code\r\n") );
	wprintf( TEXT("\r\n"));
	wprintf( TEXT(" e.g.: sim_change_pin                 ; display this message\r\n") );
	wprintf( TEXT("     : sim_change_pin -h              ; display this message\r\n") );
	wprintf( TEXT("     : sim_change_pin -o 1234 -n 8888 ; change the PIN code from 1234 to 8888\r\n") );
}


BOOL getopt(int argc, _TCHAR* argv[], _TCHAR option, _TCHAR** value)
{
	int		i;
	BOOL	bolFound = FALSE;

	for (i = 1; (i < argc); i++)
	{
		if ( bolFound )
		{
			if ( value ) {
				*value = argv[i];
			}
			break;
		}
		if ((argv[i][0] == '-') || (argv[i][0] == '/'))
		{
			if ( argv[i][1] == option )
			{
				bolFound = TRUE;
			}
		}
	}

	return bolFound;
}

void RemovePINCode(void)
{
	HKEY	hKey;
	ULONG	ulRet, ulSize;
	WCHAR	value[MAX_PATH];
	DWORD dwDisposition;

	ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, 0, &hKey);
	if ( ulRet != ERROR_SUCCESS ) 
	{
		ulRet = RegCreateKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, NULL, 0, 0, NULL, &hKey, &dwDisposition);
	}
	if ( ulRet == ERROR_SUCCESS )
	{
		ulSize = sizeof(value);
		memset(value, 0, sizeof(value));
		ulRet = RegSetValueEx( hKey, TEXT("PINCode"), NULL, REG_SZ, (LPBYTE)value, wcslen(value) * sizeof(WCHAR) );
		RegFlushKey( hKey );
		RegCloseKey( hKey );
	}
}

int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR *szOldPin=NULL, *szNewPin=NULL;
	char  *csOldPin = NULL;
	char  *csNewPin = NULL;
	int nEnable;
	int nStatus;

	if ( argc==1 ) {
		PrintUsage();
		return 0;
	}

	if ( getopt(argc, argv, 'h', NULL) ) {
		PrintUsage();
		return 0;
	}

	if ( getopt(argc, argv, 'o', &szOldPin) ) {
		if ( szOldPin ) {
			csOldPin = new char[MAX_PATH];
			memset(csOldPin, 0, sizeof(csOldPin));
			//WideCharToMultiByte(CP_ACP, 0, szOldPin, NULL, csOldPin, MAX_PATH, NULL, NULL);
			wcstombs(csOldPin, szOldPin, MAX_PATH);
		}
	}

	if ( getopt(argc, argv, 'n', &szNewPin) ) {
		if ( szNewPin ) {
			csNewPin = new char[MAX_PATH];
			memset(csNewPin, 0, sizeof(csNewPin));
			//WideCharToMultiByte(CP_ACP, 0, szNewPin, NULL, csNewPin, MAX_PATH, NULL, NULL);
			wcstombs(csNewPin, szNewPin, MAX_PATH);
		}
	}

	if ( szOldPin == NULL || szNewPin==NULL)
	{
		PrintUsage();
		if ( szOldPin ) {
			delete csOldPin;
		}
		if ( szNewPin ) {
			delete csNewPin;
		}
		return -1;
	}

	int h = cellular_modem_open();
	if ( h != 0 && h != -1 ) 
	{
		nStatus = cellular_modem_sim_get_sim_card_status(h);
		if ( nStatus == 1 ) {
			printf("No SIM Card.\n");
		} 
		else if ( nStatus == 2 ) 
		{
			printf("Need PIN code authentication.\n");
		}
		else if ( nStatus == 3 ) 
		{
			printf("Need PUK code authentication.\n");
		}
		else if ( nStatus == 0 ) 
		{
			nEnable = cellular_modem_sim_get_pin_enable_status(h);
			if ( nEnable == 0 ) {
				cellular_modem_sim_enable_pin_code(h, csOldPin, 1);
			}
			RemovePINCode();
			if ( cellular_modem_sim_assign_pin_code(h, csOldPin, csNewPin) == 0 ) {
				printf("PIN code has changed.\n");

				if ( nEnable == 0 ) {
					cellular_modem_sim_enable_pin_code(h, csNewPin, 0);
				}
			} else {
				printf("PIN code change failed.\n");
			}
		} 

		cellular_modem_close(h);
	} else {
		printf("Error!! Module Driver did not start.\n");
	}
	if ( szOldPin ) {
		delete csOldPin;
	}
	if ( szNewPin ) {
		delete csNewPin;
	}

	return 0;
}

