// sim_enable_pin.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "moxa\devices.h"

void PrintUsage( void )
{
	wprintf( TEXT("Usage: sim_enable_pin [-h] | [-e -p pin_code | -d -p pin_code]   ; Enable or Disable the PIN authentication\r\n") );
	wprintf( TEXT("\r\n"));
	wprintf( TEXT(" e.g.: sim_enable_pin                  ; display this message\r\n") );
	wprintf( TEXT("     : sim_enable_pin -h               ; display this message\r\n") );
	wprintf( TEXT("     : sim_enable_pin -e -p pin_code   ; enable the PIN code authentication\r\n") );
	wprintf( TEXT("     : sim_enable_pin -d -p pin_code   ; disable the PIN code authentication\r\n") );
}


BOOL getopt(int argc, _TCHAR* argv[], _TCHAR option, _TCHAR** value)
{
	int		i;
	BOOL	bolFound = FALSE;

	for (i = 1; (i < argc); i++)
	{
		if ( bolFound )
		{
			if ( value ) {
				*value = argv[i];
			}
			break;
		}
		if ((argv[i][0] == '-') || (argv[i][0] == '/'))
		{
			if ( argv[i][1] == option )
			{
				bolFound = TRUE;
			}
		}
	}

	return bolFound;
}

int _tmain(int argc, _TCHAR* argv[])
{
	BOOL bolEnable=FALSE;
	int status = -1, en_status=0;
	TCHAR *szPIN = NULL;
	char  *csPIN = NULL;
	int nCount=0;

	if ( getopt(argc, argv, 'h', NULL) ) {
		PrintUsage();
		return 0;
	}
	if ( getopt(argc, argv, 'e', NULL) ) {
		bolEnable = TRUE;
	} else if ( getopt(argc, argv, 'd', NULL) ) {
		bolEnable = FALSE;
	} else {
		PrintUsage();
		return -1;
	}

	if ( getopt(argc, argv, 'p', &szPIN) ) {
		if ( szPIN ) {
			csPIN = new char[MAX_PATH];
			memset(csPIN, 0, sizeof(csPIN));
			//WideCharToMultiByte(CP_ACP, 0, szPIN, NULL, csPIN, MAX_PATH, NULL, NULL);
			wcstombs( csPIN, szPIN, MAX_PATH);
		}
	}

	if ( szPIN == NULL )
	{
		PrintUsage();
		return -1;
	}

	int h = cellular_modem_open();
	if ( h != 0 && h != -1 ) 
	{
		status = cellular_modem_sim_get_sim_card_status(h);
		switch ( status ) 
		{
		case 0:
			nCount = cellular_modem_sim_get_pin_attempt_count(h);
			if ( nCount == 1 ) 
			{
				printf("You have only ONE PIN code authentication attempt left.\n");
				printf("Are you sure about this PIN code.(Y/N) ?");
				WCHAR ch = getwchar();
				if ( ch != (WCHAR)'y' && ch != (WCHAR)'Y') {
					if ( szPIN ) {
						delete szPIN;
					}
					return -1;
				}
			}

			en_status = cellular_modem_sim_get_pin_enable_status(h);
			if ( en_status == 1 )
			{
				if ( bolEnable ) {
					printf("The PIN had already been enabled.\n");
				} else {
					if ( cellular_modem_sim_enable_pin_code(h, csPIN, 0) != 0 ) {
						printf("Disable PIN code failed.\n");
					} else {
						printf("PIN code is disabled.\n");
					}
				}
			} else {
				if ( !bolEnable ) {
					printf("The PIN had already been disabled.\n");
				} else {
					if ( cellular_modem_sim_enable_pin_code(h, csPIN, 1) != 0 ) {
						printf("Enable PIN code failed.\n");
					} else {
						printf("PIN code is enabled.\n");
					}
				}
			}
			break;
		case 1:
			printf("No SIM card.\n");
			break;
		case 2:
			printf("Need PIN code authentication.\n");
			break;
		case 3:
			printf("SIM card is locked.\n");
			break;
		default:
			break;
		}

		cellular_modem_close(h);
	}
	if ( szPIN ) {
		delete csPIN;
	}

	return 0;
}

