// sim_input_pin.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "moxa\devices.h"

void PrintUsage( void )
{
	wprintf( TEXT("Usage: sim_input_pin [-h] | [-p pin_code -s]   ; PIN code authenticattion\r\n") );
	wprintf( TEXT("\r\n"));
	wprintf( TEXT(" e.g.: sim_input_pin            ; display this message\r\n") );
	wprintf( TEXT("     : sim_input_pin -h         ; display this message\r\n") );
	wprintf( TEXT("     : sim_input_pin -p 1234    ; input PIN code 1234 for SIM card authentication\r\n") );
	wprintf( TEXT("     : sim_input_pin -p 1234 -s ; input PIN code and save it for automatic authentication at system start-up.\r\n") );
}


BOOL getopt(int argc, _TCHAR* argv[], _TCHAR option, _TCHAR** value)
{
	int		i;
	BOOL	bolFound = FALSE;

	for (i = 1; (i < argc); i++)
	{
		if ( bolFound )
		{
			if ( value ) {
				*value = argv[i];
			}
			break;
		}
		if ((argv[i][0] == '-') || (argv[i][0] == '/'))
		{
			if ( argv[i][1] == option )
			{
				bolFound = TRUE;
			}
		}
	}

	return bolFound;
}

void RemovePINCode(void)
{
	HKEY	hKey;
	ULONG	ulRet, ulSize;
	WCHAR	value[MAX_PATH];
	DWORD dwDisposition;

	ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, 0, &hKey);
	if ( ulRet != ERROR_SUCCESS ) 
	{
		ulRet = RegCreateKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, NULL, 0, 0, NULL, &hKey, &dwDisposition);
	}
	if ( ulRet == ERROR_SUCCESS )
	{
		ulSize = sizeof(value);
		memset(value, 0, sizeof(value));
		ulRet = RegSetValueEx( hKey, TEXT("PINCode"), NULL, REG_SZ, (LPBYTE)value, wcslen(value) * sizeof(WCHAR) );
		RegFlushKey( hKey );
		RegCloseKey( hKey );
	}
}

void SavePINCode(WCHAR *szPasswd)
{
	HKEY	hKey;
	ULONG	ulRet, ulSize;
	WCHAR	value[MAX_PATH];
		DWORD dwDisposition;

	ulRet = RegOpenKeyEx( HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, 0, &hKey);
	if ( ulRet != ERROR_SUCCESS ) 
	{
		ulRet = RegCreateKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\MOXA\\GPRS"), 0, NULL, 0, 0, NULL, &hKey, &dwDisposition);
	}
	if ( ulRet == ERROR_SUCCESS )
	{
		ulSize = sizeof(value);
		memset(value, 0, sizeof(value));
		wcscpy(value, szPasswd);
		ulRet = RegSetValueEx( hKey, TEXT("PINCode"), NULL, REG_SZ, (LPBYTE)value, wcslen(value) * sizeof(WCHAR) );
		RegFlushKey( hKey );
		RegCloseKey( hKey );
	}
}

int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR *szPasswd=NULL;
	char  *csPasswd = NULL;
	BOOL  bolSavePassword = FALSE;
	int   nCount;
	int	  nStatus;

	if ( argc==1 ) {
		PrintUsage();
		return 0;
	}
	if ( getopt(argc, argv, 'h', NULL) ) {
		PrintUsage();
		return 0;
	}
	if ( getopt(argc, argv, 'p', &szPasswd) ) {
		if ( szPasswd ) {
			csPasswd = new char[MAX_PATH];
			memset(csPasswd, 0, sizeof(csPasswd));
			//WideCharToMultiByte(CP_ACP, 0, szPasswd, NULL, csPasswd, MAX_PATH, NULL, NULL);
			wcstombs(csPasswd, szPasswd, MAX_PATH);
		}
	}
	if ( getopt(argc, argv, 's', NULL) ) {
		bolSavePassword = TRUE;
	}
	
	if ( wcslen(szPasswd) != 4 ) {
		printf("PIN code error.\n");
		return -1;
	}

	if ( szPasswd == NULL) {
		PrintUsage();
		return -1;
	}

	int h = cellular_modem_open();
	if ( h != 0 && h != -1 ) 
	{
		nStatus = cellular_modem_sim_get_sim_card_status(h);
		if ( nStatus == 2 ) 
		{
			nCount = cellular_modem_sim_get_pin_attempt_count(h);
			if ( nCount == 1 ) 
			{
				printf("You have only ONE PIN code authentication attempt left.\n");
				printf("Are you sure about this PIN code.(Y/N) ?");
				WCHAR ch = getwchar();
				if ( ch != (WCHAR)'y' && ch != (WCHAR)'Y') {
					if ( szPasswd ) {
						delete csPasswd;
					}
					return -1;
				}
			}
			RemovePINCode();
			if ( cellular_modem_sim_authenticate_pin_code(h, csPasswd) == 0 )
			{
				printf("PIN code authentication OK.\n");
				if ( bolSavePassword ) {
					SavePINCode(szPasswd);
				}
			} else {
				printf("PIN code authentication Error.\n");
			}
		} else if ( nStatus == 0 ) {
			printf("SIM card status is ready.\n");
		} else if ( nStatus == 1 ) {
			printf("No SIM card.\n");
		} else if ( nStatus == 3 ) {
			printf("SIM card is locked.\n");
		}

		cellular_modem_close(h);
	}
	if ( szPasswd ) {
		delete csPasswd;
	}

	return 0;
}

