// sim_unlock.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <commctrl.h>
#include "moxa\devices.h"

void PrintUsage( void )
{
	wprintf( TEXT("Usage: sim_unlock [-h] | [-p PUK_code -n new_pin_code]   ; Unlock the SIM card\r\n") );
	wprintf( TEXT("\r\n"));
	wprintf( TEXT(" e.g.: sim_unlock                        ; display this message\r\n") );
	wprintf( TEXT("     : sim_unlock -h                     ; display this message\r\n") );
	wprintf( TEXT("     : sim_unlock -p 12345678 -n 1234    ; input PUK code with new pin code assigned\r\n") );
}


BOOL getopt(int argc, _TCHAR* argv[], _TCHAR option, _TCHAR** value)
{
	int		i;
	BOOL	bolFound = FALSE;

	for (i = 1; (i < argc); i++)
	{
		if ( bolFound )
		{
			if ( value ) {
				*value = argv[i];
			}
			break;
		}
		if ((argv[i][0] == '-') || (argv[i][0] == '/'))
		{
			if ( argv[i][1] == option )
			{
				bolFound = TRUE;
			}
		}
	}

	return bolFound;
}

int _tmain(int argc, _TCHAR* argv[])
{
	TCHAR *szPUK=NULL;
	char  *csPUK = NULL;
	TCHAR *szPIN=NULL;
	char  *csPIN = NULL;
	int nCount;

	if ( argc==1 ) {
		PrintUsage();
		return 0;
	}
	if ( getopt(argc, argv, 'h', NULL) ) {
		PrintUsage();
		return 0;
	}
	if ( getopt(argc, argv, 'p', &szPUK) ) {
		if ( szPUK ) {
			csPUK = new char[MAX_PATH];
			memset(csPUK, 0, sizeof(csPUK));
			//WideCharToMultiByte(CP_ACP, 0, szPUK, NULL, csPUK, MAX_PATH, NULL, NULL);
			wcstombs(csPUK, szPUK, MAX_PATH);
		}
	}
	if ( getopt(argc, argv, 'n', &szPIN) ) {
		if ( szPIN ) {
			csPIN = new char[MAX_PATH];
			memset(csPIN, 0, sizeof(csPIN));
			//WideCharToMultiByte(CP_ACP, 0, szPIN, NULL, csPIN, MAX_PATH, NULL, NULL);
			wcstombs(csPIN, szPIN, MAX_PATH);
		}
	}
	if ( szPUK == NULL || szPIN == NULL )
	{
		PrintUsage();
		if ( szPUK ) {
			delete csPUK;
		}
		if ( szPIN ) {
			delete csPIN;
		}
		return -1;
	}

	int h = cellular_modem_open();
	if ( h != 0 && h != -1 ) 
	{
		if ( cellular_modem_sim_get_sim_card_status(h) == 3 ) 
		{
			nCount = cellular_modem_sim_get_pin_attempt_count(h);
			if ( nCount <= 3 ) 
			{
				printf("You have only %d PUK code authentication attempt left.\n", nCount);
				printf("Are you sure about this PUK code.(Y/N) ?");
				WCHAR ch = getwchar();
				if ( ch != (WCHAR)'y' && ch != (WCHAR)'Y') {
					if ( szPUK ) {
						delete csPUK;
					}
					if ( szPIN ) {
						delete csPIN;
					}
					return -1;
				}
			}
			if ( cellular_modem_sim_unlock_pin_code(h, csPUK, csPIN) == 0 )
			{
				printf("SIM card is unlocked!\n");
			}
			else
			{
				printf("Unlock SIM card failed!\n");
			}
		} else {
			printf("The SIM card did not lock!\n");
		}

		cellular_modem_close(h);
	}
	if ( szPUK ) {
		delete csPUK;
	}
	if ( szPIN ) {
		delete csPIN;
	}

	return 0;
}

